@@ -18,6 +18,8 @@ function Debugger(sel) {
     this.breakpointManager = new BreakpointManager();
     this.breakpointPopover = undefined;  // Which line $elt has the popover open
 
+    this.perlVarPopover = undefined; // Whick span $elt has the popover open
+
     // Keys are expression strings, values are the div they display in
     this.watchedExpressions = {};
 
@@ -28,6 +30,7 @@ function Debugger(sel) {
     this.navTabContentTemplate = Handlebars.compile( $('#nav-tab-content-template').html() );
     this.navPaneTemplate = Handlebars.compile( $('#nav-pane-template').html() );
     this.watchedExpressionTemplate = Handlebars.compile( $('#watched-expr-template').html() );
+    this.currentSubAndArgsTemplate = Handlebars.compile( $('#current-sub-and-args-template').html() );
     Handlebars.registerHelper('ifDefined', function(val, options) {
         if ((val !== undefined) && (val !== null) && (val !== '')) {
             return options.fn(this)
@@ -35,7 +38,7 @@ function Debugger(sel) {
             return options.inverse(this);
         }
     });
-    this.breakpointRightClickMenu = Handlebars.compile( $('#breakpoint-right-click-template').html() );
+    this.breakpointRightClickMenuTemplate = Handlebars.compile( $('#breakpoint-right-click-template').html() );
 
     // The step in, over, run buttons
     this.controlButtons = $('.control-button');
@@ -44,7 +47,9 @@ function Debugger(sel) {
         this.messageHandler.addHandler('program_name', this.programNameUpdated.bind(this))
                             .addHandler('termination', this.programWasTerminated.bind(this))
                             .addHandler('evalresult', this.watchExpressionResult.bind(this))
-                            .addHandler('hangup', this.hangup.bind(this));
+                            .addHandler('hangup', this.hangup.bind(this))
+                            .addHandler('getvar', this.drawPerlVarPopover.bind(this));
+
 
         this.getProgramName();
         this.getStack();
@@ -67,6 +72,17 @@ function Debugger(sel) {
                     this.breakableLineRightClicked.bind(this));
         this.elt.on('click', '.breakpoint-condition,.action',
                     this.editBreakpointCondition.bind(this));
+        this.elt.on('hover', '.popup-perl-var',
+                    this.popoverPerlVar.bind(this));
+
+        this.elt.on('click', '.current-sub-and-args', function(e) {
+            var codeTable = $(e.currentTarget).parent().find('.program-code'),
+                activeLine = codeTable.find('.active'),
+                activeLineno = parseInt(activeLine.attr('data-line'));
+
+            dbg.setCurrentLineForCodeTable(codeTable, activeLineno);
+
+        });
 
         this.watchDiv.on('click', '.remove-watched-expression',
                     this.removeWatchExpression.bind(this));
@@ -210,10 +226,12 @@ function Debugger(sel) {
         var filename = $elt.closest('div.tab-pane').attr('data-filename'),
             lineno = $elt.closest('tr').attr('data-line'),
             bp = this.breakpointManager.get(filename, lineno),
-            menu = this.breakpointRightClickMenu({  condition: bp.condition,
+            menu = this.breakpointRightClickMenuTemplate({
+                                                    condition: bp.condition,
                                                     action: bp.action,
                                                     filename: filename,
-                                                    lineno: lineno});
+                                                    lineno: lineno
+                                                });
         if (this.breakpointPopover) {
             this.breakpointPopover.popover('destroy');
         }
@@ -301,6 +319,62 @@ function Debugger(sel) {
 
     };
 
+    // Event for when the user hovers over a perl variable in the code
+    this.popoverPerlVar = function(e) {
+        var $elt = $(e.currentTarget),
+            eval = $elt.attr('data-eval'),
+            stackLevelTag = $elt.closest('.tab-pane').attr('id'),
+            stackLevel = 0,
+            dbg = this;
+
+        // Remove any previous hover var elements
+        $('.hovered-perl-var').removeClass('hovered-perl-var');
+        if (this.perlVarPopover) {
+            this.perlVarPopover.popover('destroy');
+        }
+        if (e.type === 'mouseleave') {
+            this.perlVarPopover = undefined;
+            return;
+        }
+
+        if (stackLevelTag.match(/stack(\d+)/)) {
+            stackLevel = parseInt(RegExp.$1);
+        }
+
+        $elt.addClass('hovered-perl-var');
+        this.perlVarPopover = $elt;
+        $.ajax({url: 'getvar',
+                dataType: 'json',
+                data: { l: stackLevel, v: eval },
+                type: 'POST',
+                success: this.messageHandler.consumer,
+                error: this.ajaxError
+            });
+    }
+    this.drawPerlVarPopover = function(data) {
+        var p,
+            popupArgs,
+            title;
+
+        if (! this.perlVarPopover) {
+            return;
+        }
+
+        p = new PerlValue.parseFromEval(data.result);
+        popupArgs = {   trigger: 'manual',
+                        placement: 'bottom',
+                        html: true,
+                        container: dbg.elt,
+                        content: p.renderValue()
+                    };
+        title = p.renderHeader();
+        if (title && (title.length > 0)) {
+            popupArgs.title = title;
+        }
+        this.perlVarPopover.popover( popupArgs )
+                            .popover('show');
+    };
+
     this.programNameUpdated = function(name) {
         this.elt.find('li#program-name a').html(name);
     };
@@ -313,22 +387,6 @@ function Debugger(sel) {
             });
     };
 
-    this.clearStackTabs = function() {
-        // Clear the HTML elements
-        this.stackDiv.find('ul.nav').empty();
-        this.stackDiv.find('div.tab-content').empty();
-
-        // Remove saved stack codePanes already in the list
-        for (filename in this.codePanes) {
-            for (i = 0; i < this.codePanes[filename].length; i++) {
-                if (this.codePanes[filename][i].__is_stack) {
-                    this.codePanes[filename].splice(i, 1);
-                    i--;
-                }
-            }
-        }
-    };
-
     // Set the given line "active" (hightlited) and scroll it into view
     // if necessary
     this.setCurrentLineForCodeTable = function(codeTable, line) {
@@ -363,9 +421,11 @@ function Debugger(sel) {
             filename = frame ? frame.filename : '',
             line = frame ? frame.line : -1,
             tab = tabs.find('.stack-tab:nth-child('+ (i+1) +')'),
-            codePane = panes.find('.program-code-container:nth-child('+ (i+1) + ')'),
+            codePane = panes.find('.tab-pane:nth-child('+ (i+1) + ')'),
+            codeContainer = codePane.find('.program-code-container'),
             dbg = this,
-            html = '';
+            html = '',
+            subArgs = [];
 
         if (! frame) {
             // Remove a stack frame
@@ -388,12 +448,25 @@ function Debugger(sel) {
         tab.html(html);
         tab.find('a[rel="tooltip"]').tooltip();
 
+        var n;
+        for (n = 0; n < frame.args.length; n++) {
+            subArgs.push(('render' in frame.args[n]) ? frame.args[n].render() : frame.args[n] );
+        }
+
         if (isSameFrame) {
             // same package and filename for the current frame
             if (i === 0) {
                 // just update the currently active line
                 var codeTable = codePane.find('table.program-code');
                 this.setCurrentLineForCodeTable(codeTable, line);
+
+                var current = dbg.currentSubAndArgsTemplate({  subroutine: frame.subroutine,
+                                                                lineno: frame.line,
+                                                                subArgs: subArgs
+                                                            });
+                codePane.find('.current-sub-and-args')
+                        .empty()
+                        .html( current );
             }
             return;
         }
@@ -403,18 +476,27 @@ function Debugger(sel) {
                                         active: i === 0 });
         if (codePane.length === 0) {
             codePane = $(html).appendTo(panes);
-            dbg._setElementHeight.call(codePane);
+            codeContainer = codePane.find('.program-code-container'),
+            dbg._setElementHeight.call(codePane.find('.managed-height'));
         } else {
-            codePane.empty();
+            codeContainer.empty();
         }
         this.fileManager.loadFile(filename)
             .done(function($elt) {
                 var $copy = $elt.clone();
-                $copy.appendTo(codePane)
+                $copy.appendTo(codeContainer)
                     .bind('breakpoints-updated', dbg.breakpointsForPane);
-                    //.find('tr:nth-child('+line+')').addClass('active');
+
                 dbg.setCurrentLineForCodeTable($copy, line);
                 dbg.breakpointManager.markBreakpoints(filename, $copy);
+
+                var current = dbg.currentSubAndArgsTemplate({  subroutine: frame.subroutine,
+                                                                lineno: frame.line,
+                                                                subArgs: subArgs
+                                                            });
+                codePane.find('.current-sub-and-args')
+                        .empty()
+                        .html( current );
             });
     };
 
@@ -517,7 +599,7 @@ function Debugger(sel) {
     };
 
     this.ajaxError = function(xhdr, status, error) {
-        debugger;
+        // debugger;
         alert('Error from ajax call.  Status "' + status +'" error "'
                 +error+'": '+xhdr.responseText);
     };
