@@ -1,10 +1,3 @@
 package TestNothing;
 # This is loaded by some of the debugged test scripts
-3;
-
-sub a_sub {
-    6;
-    7;
-}
-
 1;
